@@ -4,10 +4,11 @@ __all__ = [
     'get_closest_topics', 'get_top_topic_words',
     'get_top_topic_docs']
 
-from typing import List, Union, Tuple
+from typing import List, Union, Tuple, Dict
 from scipy.sparse import csr
 from pandas import DataFrame, Series, concat
 from sklearn.feature_extraction.text import CountVectorizer
+from bitermplus._btm import BTM
 import numpy as np
 import scipy.special as ssp
 import tqdm
@@ -15,7 +16,7 @@ import tqdm
 
 def get_words_freqs(
         docs: Union[List[str], np.ndarray, Series],
-        **kwargs: dict) -> Tuple[csr.csr_matrix, np.ndarray]:
+        **kwargs: dict) -> Tuple[csr.csr_matrix, np.ndarray, Dict]:
     """Compute words vs documents frequency matrix.
 
     Parameters
@@ -31,34 +32,75 @@ def get_words_freqs(
     -------
     Tuple[csr.csr_matrix, np.ndarray]
         Words vs documents matrix in CSR format and vocabulary.
+
+    Example
+    -------
+    >>> import pandas as pd
+    >>> import bitermplus as btm
+
+    >>> # Loading data
+    >>> df = pd.read_csv(
+    ...     'dataset/SearchSnippets.txt.gz', header=None, names=['texts'])
+    >>> texts = df['texts'].str.strip().tolist()
+
+    >>> # Vectorizing documents, obtaining full vocabulary and biterms
+    >>> X, vocabulary, vocab_dict = btm.get_words_freqs(texts)
     """
     vec = CountVectorizer(**kwargs)
     X = vec.fit_transform(docs)
-    vocab = np.array(vec.get_feature_names())
-    return X, vocab
+    words = np.array(vec.get_feature_names())
+    return X, words, vec.vocabulary_
 
 
 def get_vectorized_docs(
-        x: Union[csr.csr_matrix, np.ndarray]) -> np.ndarray:
+        docs: Union[List[str],  np.ndarray],
+        vocab: Union[List[str], np.ndarray]) -> List[np.ndarray]:
     """Replace words with their ids in each document.
 
     Parameters
     ----------
-    x : Union[np.ndarray]
-        Words vs documents matrix as :meth:`scipy.sparse.csr_matrix`
-        or `numpy.ndarray`.
+    docs : Union[List[str],  np.ndarray]
+        Documents (iterable of strings).
+    vocab: Union[List[str], np.ndarray]
+        Vocabulary (iterable of terms).
 
     Returns
     -------
-    docs : np.ndarray
-        Vectorised documents.
+    docs : List[np.ndarray]
+        Vectorised documents (list of ``numpy.ndarray``
+        objects with terms ids).
+
+    Example
+    -------
+    >>> import pandas as pd
+    >>> import bitermplus as btm
+
+    >>> # Loading data
+    >>> df = pd.read_csv(
+    ...     'dataset/SearchSnippets.txt.gz', header=None, names=['texts'])
+    >>> texts = df['texts'].str.strip().tolist()
+
+    >>> # Vectorizing documents, obtaining full vocabulary and biterms
+    >>> X, vocabulary, vocab_dict = btm.get_words_freqs(texts)
+    >>> docs_vec = btm.get_vectorized_docs(texts, vocabulary)
     """
-    return list(map(lambda z: z.nonzero()[1].astype(int), x))
+    vocab_idx = dict(zip(vocab, range(len(vocab))))
+
+    def _parse_words(w):
+        return vocab_idx.get(w)
+
+    return list(
+        map(
+            lambda doc:
+                np.array(
+                    list(filter(None, map(_parse_words, doc.split()))),
+                    dtype=int),
+            docs))
 
 
 def get_biterms(
-        n_wd: Union[csr.csr_matrix, np.ndarray],
-        win: int = 15) -> List:
+        docs: List[np.ndarray],
+        win: int = 15) -> List[List[int]]:
     """Biterms creation routine.
 
     Parameters
@@ -71,24 +113,41 @@ def get_biterms(
 
     Returns
     -------
-    List[List]
+    List[List[int]]
         List of biterms for each document.
+
+    Example
+    -------
+    >>> import pandas as pd
+    >>> import bitermplus as btm
+
+    >>> # Loading data
+    >>> df = pd.read_csv(
+    ...     'dataset/SearchSnippets.txt.gz', header=None, names=['texts'])
+    >>> texts = df['texts'].str.strip().tolist()
+
+    >>> # Vectorizing documents, obtaining full vocabulary and biterms
+    >>> X, vocabulary, vocab_dict = btm.get_words_freqs(texts)
+    >>> docs_vec = btm.get_vectorized_docs(texts, vocabulary)
+    >>> biterms = btm.get_biterms(docs_vec)
     """
     biterms = []
-    for a in n_wd:
+    for doc in docs:
         doc_biterms = []
-        words = np.nonzero(a)[1]
-        for i in range(len(words)-1):
-            for j in range(i+1, min(i + win, len(words))):
-                wi = min(words[i], words[j])
-                wj = max(words[i], words[j])
+        doc_len = len(doc)
+        if doc_len < 2:
+            continue
+        for i in range(doc_len-1):
+            for j in range(i+1, min(i + win, doc_len)):
+                wi = min(doc[i], doc[j])
+                wj = max(doc[i], doc[j])
                 doc_biterms.append([wi, wj])
         biterms.append(doc_biterms)
     return biterms
 
 
 def get_closest_topics(
-        *matrices: List[Union[np.ndarray, DataFrame]],
+        *matrices: List[np.ndarray],
         ref: int = 0,
         method: str = "klb",
         thres: float = 0.9,
@@ -98,9 +157,9 @@ def get_closest_topics(
 
     Parameters
     ----------
-    *matrices : List[Union[DataFrame, np.ndarray]]
+    *matrices : List[np.ndarray]
         Sequence of topics vs words matrices (T x W).
-        This matrix can be accessed using ``matrix_words_topics_``
+        This matrix can be accessed using ``matrix_topics_words_``
         model attribute.
     ref : int = 0
         Index of reference matrix (zero-based indexing).
@@ -128,8 +187,9 @@ def get_closest_topics(
 
     Example
     -------
+    >>> # `models` must be an iterable of fitted BTM models
     >>> closest_topics, kldiv = btm.get_closest_topics(
-            *list(map(lambda x: x.matrix_words_topics_, models)))
+    ...     *list(map(lambda x: x.matrix_topics_words_, models)))
     """
     matrices_num = len(matrices)
     ref = matrices_num - 1 if ref >= matrices_num else ref
@@ -222,12 +282,16 @@ def get_stable_topics(
         Filtered distance values corresponding to the matrix of
         the closest topics.
 
+    See Also
+    --------
+    bitermplus.get_closest_topics
+
     Example
     -------
     >>> closest_topics, kldiv = btm.get_closest_topics(
-            *list(map(lambda x: x.matrix_words_topics_, models)))
+    ...     *list(map(lambda x: x.matrix_topics_words_, models)))
     >>> stable_topics, stable_kldiv = btm.get_stable_topics(
-            closest_topics, kldiv)
+    ...     closest_topics, kldiv)
     """
     dist_arr = np.asarray(dist)
     dist_norm = 1 - (dist_arr / dist_arr.max())
@@ -238,9 +302,9 @@ def get_stable_topics(
 
 
 def get_top_topic_words(
-        model,
+        model: BTM,
         words_num: int = 20,
-        topics_idx: Union[List, np.ndarray] = None) -> DataFrame:
+        topics_idx: Union[List[int], np.ndarray] = None) -> DataFrame:
     """Select top topic words from a fitted model.
 
     Parameters
@@ -257,6 +321,14 @@ def get_top_topic_words(
     -------
     DataFrame
         Words with highest probabilities in all selected topics.
+
+    Example
+    -------
+    >>> stable_topics = [0, 3, 10, 12, 18, 21]
+    >>> top_words = btm.get_top_topic_words(
+    ...     model,
+    ...     words_num=100,
+    ...     topics_idx=stable_topics)
     """
     def _select_words(model, topic_id: int):
         ps = model.matrix_topics_words_[topic_id, :]
@@ -275,14 +347,14 @@ def get_top_topic_docs(
         docs: Union[List[str], np.ndarray],
         p_zd: np.ndarray,
         docs_num: int = 20,
-        topics_idx: Union[List, np.ndarray] = None) -> DataFrame:
+        topics_idx: Union[List[int], np.ndarray] = None) -> DataFrame:
     """Select top topic docs from a fitted model.
 
     Parameters
     ----------
     docs : Union[List[str], np.ndarray]
-        List of documents.
-    p_zd : np.ndarray,
+        Iterable of documents (e.g. list of strings).
+    p_zd : np.ndarray
         Documents vs topics probabilities matrix.
     docs_num : int = 20
         The number of documents to select.
@@ -294,6 +366,14 @@ def get_top_topic_docs(
     -------
     DataFrame
         Documents with highest probabilities in all selected topics.
+
+    Example
+    -------
+    >>> top_docs = btm.get_top_topic_docs(
+    ...     texts,
+    ...     p_zd,
+    ...     docs_num=100,
+    ...     topics_idx=[1,2,3,4])
     """
     def _select_docs(docs, p_zd, topic_id: int):
         ps = p_zd[:, topic_id]
